@@ -58,37 +58,40 @@ import posixpath
 import re
 import sys
 import warnings
+from os import path
 from types import ModuleType
-from typing import List, cast
+from typing import Any, Dict, List, Tuple
+from typing import cast
 
 from docutils import nodes
+from docutils.nodes import Element, Node, system_message
 from docutils.parsers.rst import directives
-from docutils.parsers.rst.states import RSTStateMachine, state_classes
+from docutils.parsers.rst.states import Inliner, RSTStateMachine, Struct, state_classes
 from docutils.statemachine import StringList
 
 import sphinx
 from sphinx import addnodes
+from sphinx.application import Sphinx
 from sphinx.deprecation import RemovedInSphinx40Warning
+from sphinx.environment import BuildEnvironment
 from sphinx.environment.adapters.toctree import TocTree
-from sphinx.ext.autodoc import get_documenters
+from sphinx.ext.autodoc import Documenter, get_documenters
 from sphinx.ext.autodoc.directive import DocumenterBridge, Options
-from sphinx.ext.autodoc.importer import import_module, mock
+from sphinx.ext.autodoc.importer import import_module
+from sphinx.ext.autodoc.mock import mock
 from sphinx.locale import __
 from sphinx.pycode import ModuleAnalyzer, PycodeError
 from sphinx.util import import_object, rst, logging
 from sphinx.util.docutils import (
-    NullReporter, SphinxDirective, new_document, switch_source_input
+    NullReporter, SphinxDirective, SphinxRole, new_document, switch_source_input
 )
 from sphinx.util.matching import Matcher
+from sphinx.writers.html import HTMLTranslator
 
 if False:
     # For type annotation
-    from typing import Any, Dict, Tuple, Type  # NOQA
-    from docutils.parsers.rst.states import Inliner  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
-    from sphinx.ext.autodoc import Documenter  # NOQA
-    from sphinx.writers.html import HTMLTranslator  # NOQA
+    from typing import Type  # for python3.5.1
+
 
 logger = logging.getLogger(__name__)
 
@@ -103,16 +106,14 @@ class autosummary_toc(nodes.comment):
     pass
 
 
-def process_autosummary_toc(app, doctree):
-    # type: (Sphinx, nodes.document) -> None
+def process_autosummary_toc(app: Sphinx, doctree: nodes.document) -> None:
     """Insert items described in autosummary:: to the TOC tree, but do
     not generate the toctree:: list.
     """
     env = app.builder.env
     crawled = {}
 
-    def crawl_toc(node, depth=1):
-        # type: (nodes.Element, int) -> None
+    def crawl_toc(node: Element, depth: int = 1) -> None:
         crawled[node] = True
         for j, subnode in enumerate(node):
             try:
@@ -129,14 +130,12 @@ def process_autosummary_toc(app, doctree):
     crawl_toc(doctree)
 
 
-def autosummary_toc_visit_html(self, node):
-    # type: (nodes.NodeVisitor, autosummary_toc) -> None
+def autosummary_toc_visit_html(self: nodes.NodeVisitor, node: autosummary_toc) -> None:
     """Hide autosummary toctree list in HTML output."""
     raise nodes.SkipNode
 
 
-def autosummary_noop(self, node):
-    # type: (nodes.NodeVisitor, nodes.Node) -> None
+def autosummary_noop(self: nodes.NodeVisitor, node: Node) -> None:
     pass
 
 
@@ -146,8 +145,7 @@ class autosummary_table(nodes.comment):
     pass
 
 
-def autosummary_table_visit_html(self, node):
-    # type: (HTMLTranslator, autosummary_table) -> None
+def autosummary_table_visit_html(self: HTMLTranslator, node: autosummary_table) -> None:
     """Make the first column of the table non-breaking."""
     try:
         table = cast(nodes.table, node[0])
@@ -172,13 +170,14 @@ _app = None  # type: Sphinx
 
 
 class FakeDirective(DocumenterBridge):
-    def __init__(self):
-        # type: () -> None
-        super().__init__({}, None, Options(), 0)  # type: ignore
+    def __init__(self) -> None:
+        settings = Struct(tab_width=8)
+        document = Struct(settings=settings)
+        state = Struct(document=document)
+        super().__init__({}, None, Options(), 0, state)  # type: ignore
 
 
-def get_documenter(app, obj, parent):
-    # type: (Sphinx, Any, Any) -> Type[Documenter]
+def get_documenter(app: Sphinx, obj: Any, parent: Any) -> "Type[Documenter]":
     """Get an autodoc.Documenter class suitable for documenting the given
     object.
 
@@ -232,10 +231,9 @@ class Autosummary(SphinxDirective):
         'template': directives.unchanged,
     }
 
-    def run(self):
-        # type: () -> List[nodes.Node]
+    def run(self) -> List[Node]:
         self.bridge = DocumenterBridge(self.env, self.state.document.reporter,
-                                       Options(), self.lineno)
+                                       Options(), self.lineno, self.state)
 
         names = [x.strip().split()[0] for x in self.content
                  if x.strip() and re.search(r'^[~a-zA-Z_]', x.strip()[0])]
@@ -252,24 +250,30 @@ class Autosummary(SphinxDirective):
                 docname = posixpath.join(tree_prefix, real_name)
                 docname = posixpath.normpath(posixpath.join(dirname, docname))
                 if docname not in self.env.found_docs:
+                    location = self.state_machine.get_source_and_line(self.lineno)
                     if excluded(self.env.doc2path(docname, None)):
-                        logger.warning(__('toctree references excluded document %r'), docname)
+                        msg = __('autosummary references excluded document %r. Ignored.')
                     else:
-                        logger.warning(__('toctree references unknown document %r'), docname)
+                        msg = __('autosummary: stub file not found %r. '
+                                 'Check your autosummary_generate setting.')
+
+                    logger.warning(msg, real_name, location=location)
+                    continue
+
                 docnames.append(docname)
 
-            tocnode = addnodes.toctree()
-            tocnode['includefiles'] = docnames
-            tocnode['entries'] = [(None, docn) for docn in docnames]
-            tocnode['maxdepth'] = -1
-            tocnode['glob'] = None
+            if docnames:
+                tocnode = addnodes.toctree()
+                tocnode['includefiles'] = docnames
+                tocnode['entries'] = [(None, docn) for docn in docnames]
+                tocnode['maxdepth'] = -1
+                tocnode['glob'] = None
 
-            nodes.append(autosummary_toc('', '', tocnode))
+                nodes.append(autosummary_toc('', '', tocnode))
 
         return nodes
 
-    def get_items(self, names):
-        # type: (List[str]) -> List[Tuple[str, str, str, str]]
+    def get_items(self, names: List[str]) -> List[Tuple[str, str, str, str]]:
         """Try to import the given names, and return a list of
         ``[(name, signature, summary_string, real_name), ...]``.
         """
@@ -328,7 +332,12 @@ class Autosummary(SphinxDirective):
 
             # -- Grab the signature
 
-            sig = documenter.format_signature()
+            try:
+                sig = documenter.format_signature(show_annotation=False)
+            except TypeError:
+                # the documenter does not support ``show_annotation`` option
+                sig = documenter.format_signature()
+
             if not sig:
                 sig = ''
             else:
@@ -344,8 +353,7 @@ class Autosummary(SphinxDirective):
 
         return items
 
-    def get_table(self, items):
-        # type: (List[Tuple[str, str, str, str]]) -> List[nodes.Node]
+    def get_table(self, items: List[Tuple[str, str, str, str]]) -> List[Node]:
         """Generate a proper list of table nodes for autosummary:: directive.
 
         *items* is a list produced by :meth:`get_items`.
@@ -363,8 +371,7 @@ class Autosummary(SphinxDirective):
         body = nodes.tbody('')
         group.append(body)
 
-        def append_row(*column_texts):
-            # type: (str) -> None
+        def append_row(*column_texts: str) -> None:
             row = nodes.row('')
             source, line = self.state_machine.get_source_and_line()
             for text in column_texts:
@@ -392,42 +399,36 @@ class Autosummary(SphinxDirective):
 
         return [table_spec, table]
 
-    def warn(self, msg):
-        # type: (str) -> None
+    def warn(self, msg: str) -> None:
         warnings.warn('Autosummary.warn() is deprecated',
                       RemovedInSphinx40Warning, stacklevel=2)
         logger.warning(msg)
 
     @property
-    def genopt(self):
-        # type: () -> Options
+    def genopt(self) -> Options:
         warnings.warn('Autosummary.genopt is deprecated',
                       RemovedInSphinx40Warning, stacklevel=2)
         return self.bridge.genopt
 
     @property
-    def warnings(self):
-        # type: () -> List[nodes.Node]
+    def warnings(self) -> List[Node]:
         warnings.warn('Autosummary.warnings is deprecated',
                       RemovedInSphinx40Warning, stacklevel=2)
         return []
 
     @property
-    def result(self):
-        # type: () -> StringList
+    def result(self) -> StringList:
         warnings.warn('Autosummary.result is deprecated',
                       RemovedInSphinx40Warning, stacklevel=2)
         return self.bridge.result
 
 
-def strip_arg_typehint(s):
-    # type: (str) -> str
+def strip_arg_typehint(s: str) -> str:
     """Strip a type hint from argument definition."""
     return s.split(':')[0].strip()
 
 
-def mangle_signature(sig, max_chars=30):
-    # type: (str, int) -> str
+def mangle_signature(sig: str, max_chars: int = 30) -> str:
     """Reformat a function signature to a more compact form."""
     # Strip return type annotation
     s = re.sub(r"\)\s*->\s.*$", ")", sig)
@@ -435,16 +436,26 @@ def mangle_signature(sig, max_chars=30):
     # Remove parenthesis
     s = re.sub(r"^\((.*)\)$", r"\1", s).strip()
 
-    # Strip strings (which can contain things that confuse the code below)
-    s = re.sub(r"\\\\", "", s)
-    s = re.sub(r"\\'", "", s)
-    s = re.sub(r"'[^']*'", "", s)
+    # Strip literals (which can contain things that confuse the code below)
+    s = re.sub(r"\\\\", "", s)      # escaped backslash (maybe inside string)
+    s = re.sub(r"\\'", "", s)       # escaped single quote
+    s = re.sub(r'\\"', "", s)       # escaped double quote
+    s = re.sub(r"'[^']*'", "", s)   # string literal (w/ single quote)
+    s = re.sub(r'"[^"]*"', "", s)   # string literal (w/ double quote)
+
+    # Strip complex objects (maybe default value of arguments)
+    while re.search(r'\([^)]*\)', s):   # contents of parenthesis (ex. NamedTuple(attr=...))
+        s = re.sub(r'\([^)]*\)', '', s)
+    while re.search(r'<[^>]*>', s):     # contents of angle brackets (ex. <object>)
+        s = re.sub(r'<[^>]*>', '', s)
+    while re.search(r'{[^}]*}', s):     # contents of curly brackets (ex. dict)
+        s = re.sub(r'{[^}]*}', '', s)
 
     # Parse the signature to arguments + options
     args = []  # type: List[str]
     opts = []  # type: List[str]
 
-    opt_re = re.compile(r"^(.*, |)([a-zA-Z0-9_*]+)=")
+    opt_re = re.compile(r"^(.*, |)([a-zA-Z0-9_*]+)\s*=\s*")
     while s:
         m = opt_re.search(s)
         if not m:
@@ -474,8 +485,7 @@ def mangle_signature(sig, max_chars=30):
     return "(%s)" % sig
 
 
-def extract_summary(doc, document):
-    # type: (List[str], Any) -> str
+def extract_summary(doc: List[str], document: Any) -> str:
     """Extract summary from docstring."""
 
     # Skip a blank lines at the top
@@ -523,8 +533,8 @@ def extract_summary(doc, document):
     return summary
 
 
-def limited_join(sep, items, max_chars=30, overflow_marker="..."):
-    # type: (str, List[str], int, str) -> str
+def limited_join(sep: str, items: List[str], max_chars: int = 30,
+                 overflow_marker: str = "...") -> str:
     """Join a number of strings to one, limiting the length to *max_chars*.
 
     If the string overflows this limit, replace the last fitting item by
@@ -550,8 +560,7 @@ def limited_join(sep, items, max_chars=30, overflow_marker="..."):
 
 # -- Importing items -----------------------------------------------------------
 
-def get_import_prefixes_from_env(env):
-    # type: (BuildEnvironment) -> List[str]
+def get_import_prefixes_from_env(env: BuildEnvironment) -> List[str]:
     """
     Obtain current Python import prefixes (for `import_by_name`)
     from ``document.env``
@@ -572,8 +581,7 @@ def get_import_prefixes_from_env(env):
     return prefixes
 
 
-def import_by_name(name, prefixes=[None]):
-    # type: (str, List[str]) -> Tuple[str, Any, Any, str]
+def import_by_name(name: str, prefixes: List[str] = [None]) -> Tuple[str, Any, Any, str]:
     """Import a Python object that has the given *name*, under one of the
     *prefixes*.  The first name that succeeds is used.
     """
@@ -591,8 +599,7 @@ def import_by_name(name, prefixes=[None]):
     raise ImportError('no module named %s' % ' or '.join(tried))
 
 
-def _import_by_name(name):
-    # type: (str) -> Tuple[Any, Any, str]
+def _import_by_name(name: str) -> Tuple[Any, Any, str]:
     """Import a Python object given its full name."""
     try:
         name_parts = name.split('.')
@@ -635,13 +642,15 @@ def _import_by_name(name):
 
 # -- :autolink: (smart default role) -------------------------------------------
 
-def autolink_role(typ, rawtext, etext, lineno, inliner, options={}, content=[]):
-    # type: (str, str, str, int, Inliner, Dict, List[str]) -> Tuple[List[nodes.Node], List[nodes.system_message]]  # NOQA
+def autolink_role(typ: str, rawtext: str, etext: str, lineno: int, inliner: Inliner,
+                  options: Dict = {}, content: List[str] = []
+                  ) -> Tuple[List[Node], List[system_message]]:
     """Smart linking role.
 
     Expands to ':obj:`text`' if `text` is an object that can be imported;
     otherwise expands to '*text*'.
     """
+    warnings.warn('autolink_role() is deprecated.', RemovedInSphinx40Warning)
     env = inliner.document.settings.env
     pyobj_role = env.get_domain('py').role('obj')
     objects, msg = pyobj_role('obj', rawtext, etext, lineno, inliner, options, content)
@@ -660,10 +669,35 @@ def autolink_role(typ, rawtext, etext, lineno, inliner, options={}, content=[]):
     return objects, msg
 
 
-def get_rst_suffix(app):
-    # type: (Sphinx) -> str
-    def get_supported_format(suffix):
-        # type: (str) -> Tuple[str, ...]
+class AutoLink(SphinxRole):
+    """Smart linking role.
+
+    Expands to ':obj:`text`' if `text` is an object that can be imported;
+    otherwise expands to '*text*'.
+    """
+    def run(self) -> Tuple[List[Node], List[system_message]]:
+        pyobj_role = self.env.get_domain('py').role('obj')
+        objects, errors = pyobj_role('obj', self.rawtext, self.text, self.lineno,
+                                     self.inliner, self.options, self.content)
+        if errors:
+            return objects, errors
+
+        assert len(objects) == 1
+        pending_xref = cast(addnodes.pending_xref, objects[0])
+        try:
+            # try to import object by name
+            prefixes = get_import_prefixes_from_env(self.env)
+            import_by_name(pending_xref['reftarget'], prefixes)
+        except ImportError:
+            literal = cast(nodes.literal, pending_xref[0])
+            objects[0] = nodes.emphasis(self.rawtext, literal.astext(),
+                                        classes=literal['classes'])
+
+        return objects, errors
+
+
+def get_rst_suffix(app: Sphinx) -> str:
+    def get_supported_format(suffix: str) -> Tuple[str, ...]:
         parser_class = app.registry.get_source_parsers().get(suffix)
         if parser_class is None:
             return ('restructuredtext',)
@@ -679,39 +713,42 @@ def get_rst_suffix(app):
     return None
 
 
-def process_generate_options(app):
-    # type: (Sphinx) -> None
+def process_generate_options(app: Sphinx) -> None:
     genfiles = app.config.autosummary_generate
 
-    if genfiles and not hasattr(genfiles, '__len__'):
+    if genfiles is True:
         env = app.builder.env
         genfiles = [env.doc2path(x, base=None) for x in env.found_docs
                     if os.path.isfile(env.doc2path(x))]
+    else:
+        ext = list(app.config.source_suffix)
+        genfiles = [genfile + (not genfile.endswith(tuple(ext)) and ext[0] or '')
+                    for genfile in genfiles]
+
+        for entry in genfiles[:]:
+            if not path.isfile(path.join(app.srcdir, entry)):
+                logger.warning(__('autosummary_generate: file not found: %s'), entry)
+                genfiles.remove(entry)
 
     if not genfiles:
         return
 
-    from sphinx.ext.autosummary.generate import generate_autosummary_docs
-
-    ext = list(app.config.source_suffix)
-    genfiles = [genfile + (not genfile.endswith(tuple(ext)) and ext[0] or '')
-                for genfile in genfiles]
-
     suffix = get_rst_suffix(app)
     if suffix is None:
         logger.warning(__('autosummary generats .rst files internally. '
                           'But your source_suffix does not contain .rst. Skipped.'))
         return
 
+    from sphinx.ext.autosummary.generate import generate_autosummary_docs
+
+    imported_members = app.config.autosummary_imported_members
     with mock(app.config.autosummary_mock_imports):
         generate_autosummary_docs(genfiles, builder=app.builder,
-                                  warn=logger.warning, info=logger.info,
                                   suffix=suffix, base_path=app.srcdir,
-                                  app=app)
+                                  app=app, imported_members=imported_members)
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[str, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     # I need autodoc
     app.setup_extension('sphinx.ext.autodoc')
     app.add_node(autosummary_toc,
@@ -727,11 +764,12 @@ def setup(app):
                  man=(autosummary_noop, autosummary_noop),
                  texinfo=(autosummary_noop, autosummary_noop))
     app.add_directive('autosummary', Autosummary)
-    app.add_role('autolink', autolink_role)
+    app.add_role('autolink', AutoLink())
     app.connect('doctree-read', process_autosummary_toc)
     app.connect('builder-inited', process_generate_options)
     app.add_config_value('autosummary_generate', [], True, [bool])
     app.add_config_value('autosummary_mock_imports',
                          lambda config: config.autodoc_mock_imports, 'env')
+    app.add_config_value('autosummary_imported_members', [], False, [bool])
 
     return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
