@@ -17,11 +17,16 @@ import tempfile
 from hashlib import sha1
 from os import path
 from subprocess import CalledProcessError, PIPE
+from typing import Any, Dict, List, Tuple
 
 from docutils import nodes
+from docutils.nodes import Element
 
 import sphinx
 from sphinx import package_dir
+from sphinx.application import Sphinx
+from sphinx.builders import Builder
+from sphinx.config import Config
 from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
 from sphinx.errors import SphinxError
 from sphinx.locale import _, __
@@ -30,14 +35,7 @@ from sphinx.util.math import get_node_equation_number, wrap_displaymath
 from sphinx.util.osutil import ensuredir
 from sphinx.util.png import read_png_depth, write_png_depth
 from sphinx.util.template import LaTeXRenderer
-
-if False:
-    # For type annotation
-    from typing import Any, Dict, List, Tuple, Union  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-    from sphinx.builders import Builder  # NOQA
-    from sphinx.config import Config  # NOQA
-    from sphinx.writers.html import HTMLTranslator  # NOQA
+from sphinx.writers.html import HTMLTranslator
 
 logger = logging.getLogger(__name__)
 
@@ -47,8 +45,7 @@ templates_path = path.join(package_dir, 'templates', 'imgmath')
 class MathExtError(SphinxError):
     category = 'Math extension error'
 
-    def __init__(self, msg, stderr=None, stdout=None):
-        # type: (str, bytes, bytes) -> None
+    def __init__(self, msg: str, stderr: bytes = None, stdout: bytes = None) -> None:
         if stderr:
             msg += '\n[stderr]\n' + stderr.decode(sys.getdefaultencoding(), 'replace')
         if stdout:
@@ -90,15 +87,38 @@ DOC_BODY_PREVIEW = r'''
 '''
 
 depth_re = re.compile(br'\[\d+ depth=(-?\d+)\]')
+depthsvg_re = re.compile(br'.*, depth=(.*)pt')
+depthsvgcomment_re = re.compile(r'<!-- DEPTH=(-?\d+) -->')
 
 
-def generate_latex_macro(math, config, confdir=''):
-    # type: (str, Config, str) -> str
+def read_svg_depth(filename: str) -> int:
+    """Read the depth from comment at last line of SVG file
+    """
+    with open(filename, 'r') as f:
+        for line in f:
+            pass
+        # Only last line is checked
+        matched = depthsvgcomment_re.match(line)
+        if matched:
+            return int(matched.group(1))
+        return None
+
+
+def write_svg_depth(filename: str, depth: int) -> None:
+    """Write the depth to SVG file as a comment at end of file
+    """
+    with open(filename, 'a') as f:
+        f.write('\n<!-- DEPTH=%s -->' % depth)
+
+
+def generate_latex_macro(image_format: str,
+                         math: str, config: Config, confdir: str = '') -> str:
     """Generate LaTeX macro."""
     variables = {
         'fontsize': config.imgmath_font_size,
         'baselineskip': int(round(config.imgmath_font_size * 1.2)),
         'preamble': config.imgmath_latex_preamble,
+        'tightpage': '' if image_format == 'png' else ',tightpage',
         'math': math
     }
 
@@ -115,8 +135,7 @@ def generate_latex_macro(math, config, confdir=''):
     return LaTeXRenderer(templates_path).render(template_name, variables)
 
 
-def ensure_tempdir(builder):
-    # type: (Builder) -> str
+def ensure_tempdir(builder: Builder) -> str:
     """Create temporary directory.
 
     use only one tempdir per build -- the use of a directory is cleaner
@@ -129,8 +148,7 @@ def ensure_tempdir(builder):
     return builder._imgmath_tempdir  # type: ignore
 
 
-def compile_math(latex, builder):
-    # type: (str, Builder) -> str
+def compile_math(latex: str, builder: Builder) -> str:
     """Compile LaTeX macros for math to DVI."""
     tempdir = ensure_tempdir(builder)
     filename = path.join(tempdir, 'math.tex')
@@ -157,8 +175,7 @@ def compile_math(latex, builder):
         raise MathExtError('latex exited with error', exc.stderr, exc.stdout)
 
 
-def convert_dvi_to_image(command, name):
-    # type: (List[str], str) -> Tuple[bytes, bytes]
+def convert_dvi_to_image(command: List[str], name: str) -> Tuple[bytes, bytes]:
     """Convert DVI file to specific image format."""
     try:
         ret = subprocess.run(command, stdout=PIPE, stderr=PIPE, check=True)
@@ -172,8 +189,7 @@ def convert_dvi_to_image(command, name):
         raise MathExtError('%s exited with error' % name, exc.stderr, exc.stdout)
 
 
-def convert_dvi_to_png(dvipath, builder):
-    # type: (str, Builder) -> Tuple[str, int]
+def convert_dvi_to_png(dvipath: str, builder: Builder) -> Tuple[str, int]:
     """Convert DVI file to PNG image."""
     tempdir = ensure_tempdir(builder)
     filename = path.join(tempdir, 'math.png')
@@ -199,8 +215,7 @@ def convert_dvi_to_png(dvipath, builder):
     return filename, depth
 
 
-def convert_dvi_to_svg(dvipath, builder):
-    # type: (str, Builder) -> Tuple[str, int]
+def convert_dvi_to_svg(dvipath: str, builder: Builder) -> Tuple[str, int]:
     """Convert DVI file to SVG image."""
     tempdir = ensure_tempdir(builder)
     filename = path.join(tempdir, 'math.svg')
@@ -210,12 +225,21 @@ def convert_dvi_to_svg(dvipath, builder):
     command.extend(builder.config.imgmath_dvisvgm_args)
     command.append(dvipath)
 
-    convert_dvi_to_image(command, name)
-    return filename, None
+    stdout, stderr = convert_dvi_to_image(command, name)
+
+    depth = None
+    if builder.config.imgmath_use_preview:
+        for line in stderr.splitlines():  # not stdout !
+            matched = depthsvg_re.match(line)
+            if matched:
+                depth = round(float(matched.group(1)) * 100 / 72.27)  # assume 100ppi
+                write_svg_depth(filename, depth)
+                break
+
+    return filename, depth
 
 
-def render_math(self, math):
-    # type: (HTMLTranslator, str) -> Tuple[str, int]
+def render_math(self: HTMLTranslator, math: str) -> Tuple[str, int]:
     """Render the LaTeX math expression *math* using latex and dvipng or
     dvisvgm.
 
@@ -233,13 +257,19 @@ def render_math(self, math):
     if image_format not in SUPPORT_FORMAT:
         raise MathExtError('imgmath_image_format must be either "png" or "svg"')
 
-    latex = generate_latex_macro(math, self.builder.config, self.builder.confdir)
+    latex = generate_latex_macro(image_format,
+                                 math,
+                                 self.builder.config,
+                                 self.builder.confdir)
 
     filename = "%s.%s" % (sha1(latex.encode()).hexdigest(), image_format)
     relfn = posixpath.join(self.builder.imgpath, 'math', filename)
     outfn = path.join(self.builder.outdir, self.builder.imagedir, 'math', filename)
     if path.isfile(outfn):
-        depth = read_png_depth(outfn)
+        if image_format == 'png':
+            depth = read_png_depth(outfn)
+        elif image_format == 'svg':
+            depth = read_svg_depth(outfn)
         return relfn, depth
 
     # if latex or dvipng (dvisvgm) has failed once, don't bother to try again
@@ -271,8 +301,7 @@ def render_math(self, math):
     return relfn, depth
 
 
-def cleanup_tempdir(app, exc):
-    # type: (Sphinx, Exception) -> None
+def cleanup_tempdir(app: Sphinx, exc: Exception) -> None:
     if exc:
         return
     if not hasattr(app.builder, '_imgmath_tempdir'):
@@ -283,15 +312,13 @@ def cleanup_tempdir(app, exc):
         pass
 
 
-def get_tooltip(self, node):
-    # type: (HTMLTranslator, Union[nodes.math, nodes.math_block]) -> str
+def get_tooltip(self: HTMLTranslator, node: Element) -> str:
     if self.builder.config.imgmath_add_tooltips:
         return ' alt="%s"' % self.encode(node.astext()).strip()
     return ''
 
 
-def html_visit_math(self, node):
-    # type: (HTMLTranslator, nodes.math) -> None
+def html_visit_math(self: HTMLTranslator, node: nodes.math) -> None:
     try:
         fname, depth = render_math(self, '$' + node.astext() + '$')
     except MathExtError as exc:
@@ -313,8 +340,7 @@ def html_visit_math(self, node):
     raise nodes.SkipNode
 
 
-def html_visit_displaymath(self, node):
-    # type: (HTMLTranslator, nodes.math_block) -> None
+def html_visit_displaymath(self: HTMLTranslator, node: nodes.math_block) -> None:
     if node['nowrap']:
         latex = node.astext()
     else:
@@ -354,8 +380,7 @@ deprecated_alias('sphinx.ext.imgmath',
                  RemovedInSphinx40Warning)
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[str, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.add_html_math_renderer('imgmath',
                                (html_visit_math, None),
                                (html_visit_displaymath, None))
