@@ -398,7 +398,7 @@ def inspect_main(argv: List[str]) -> None:
 
 
 if __name__ == '__main__':
-    import logging  # type: ignore
-    logging.basicConfig()
+    import logging as _logging
+    _logging.basicConfig()
 
     inspect_main(argv=sys.argv[1:])
