@@ -9,26 +9,24 @@
 """
 
 import warnings
+from typing import Callable, List, Tuple
 
 from docutils import nodes
+from docutils.nodes import Element, Node
 from docutils.parsers.rst.roles import math_role as math_role_base
 
 from sphinx.addnodes import math, math_block as displaymath  # NOQA  # to keep compatibility
+from sphinx.application import Sphinx
 from sphinx.builders.latex.nodes import math_reference as eqref  # NOQA  # to keep compatibility
 from sphinx.deprecation import RemovedInSphinx30Warning
 from sphinx.directives.patches import MathDirective as MathDirectiveBase
 from sphinx.domains.math import MathDomain  # NOQA  # to keep compatibility
 from sphinx.domains.math import MathReferenceRole as EqXRefRole  # NOQA  # to keep compatibility
-
-if False:
-    # For type annotation
-    from typing import Callable, Tuple  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-    from sphinx.writers.html import HTMLTranslator  # NOQA
+from sphinx.writers.html import HTMLTranslator
 
 
 class MathDirective(MathDirectiveBase):
-    def run(self):
+    def run(self) -> List[Node]:
         warnings.warn('sphinx.ext.mathbase.MathDirective is moved to '
                       'sphinx.directives.patches package.',
                       RemovedInSphinx30Warning, stacklevel=2)
@@ -42,8 +40,7 @@ def math_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
     return math_role_base(role, rawtext, text, lineno, inliner, options, content)
 
 
-def get_node_equation_number(writer, node):
-    # type: (HTMLTranslator, nodes.math_block) -> str
+def get_node_equation_number(writer: HTMLTranslator, node: nodes.math_block) -> str:
     warnings.warn('sphinx.ext.mathbase.get_node_equation_number() is moved to '
                   'sphinx.util.math package.',
                   RemovedInSphinx30Warning, stacklevel=2)
@@ -51,8 +48,7 @@ def get_node_equation_number(writer, node):
     return get_node_equation_number(writer, node)
 
 
-def wrap_displaymath(text, label, numbering):
-    # type: (str, str, bool) -> str
+def wrap_displaymath(text: str, label: str, numbering: bool) -> str:
     warnings.warn('sphinx.ext.mathbase.wrap_displaymath() is moved to '
                   'sphinx.util.math package.',
                   RemovedInSphinx30Warning, stacklevel=2)
@@ -60,8 +56,7 @@ def wrap_displaymath(text, label, numbering):
     return wrap_displaymath(text, label, numbering)
 
 
-def is_in_section_title(node):
-    # type: (nodes.Element) -> bool
+def is_in_section_title(node: Element) -> bool:
     """Determine whether the node is in a section title"""
     from sphinx.util.nodes import traverse_parent
 
@@ -75,8 +70,9 @@ def is_in_section_title(node):
     return False
 
 
-def setup_math(app, htmlinlinevisitors, htmldisplayvisitors):
-    # type: (Sphinx, Tuple[Callable, Callable], Tuple[Callable, Callable]) -> None
+def setup_math(app: Sphinx,
+               htmlinlinevisitors: Tuple[Callable, Callable],
+               htmldisplayvisitors: Tuple[Callable, Callable]) -> None:
     warnings.warn('setup_math() is deprecated. '
                   'Please use app.add_html_math_renderer() instead.',
                   RemovedInSphinx30Warning, stacklevel=2)
