@@ -54,13 +54,13 @@ additional_nodes = set()  # type: Set[Type[nodes.Element]]
 def docutils_namespace() -> Generator[None, None, None]:
     """Create namespace for reST parsers."""
     try:
-        _directives = copy(directives._directives)
-        _roles = copy(roles._roles)
+        _directives = copy(directives._directives)  # type: ignore
+        _roles = copy(roles._roles)  # type: ignore
 
         yield
     finally:
-        directives._directives = _directives
-        roles._roles = _roles
+        directives._directives = _directives  # type: ignore
+        roles._roles = _roles  # type: ignore
 
         for node in list(additional_nodes):
             unregister_node(node)
@@ -69,7 +69,7 @@ def docutils_namespace() -> Generator[None, None, None]:
 
 def is_directive_registered(name: str) -> bool:
     """Check the *name* directive is already registered."""
-    return name in directives._directives
+    return name in directives._directives  # type: ignore
 
 
 def register_directive(name: str, directive: "Type[Directive]") -> None:
@@ -83,7 +83,7 @@ def register_directive(name: str, directive: "Type[Directive]") -> None:
 
 def is_role_registered(name: str) -> bool:
     """Check the *name* role is already registered."""
-    return name in roles._roles
+    return name in roles._roles  # type: ignore
 
 
 def register_role(name: str, role: RoleFunction) -> None:
@@ -97,7 +97,7 @@ def register_role(name: str, role: RoleFunction) -> None:
 
 def unregister_role(name: str) -> None:
     """Unregister a role from docutils."""
-    roles._roles.pop(name, None)
+    roles._roles.pop(name, None)  # type: ignore
 
 
 def is_node_registered(node: "Type[Element]") -> bool:
@@ -112,7 +112,7 @@ def register_node(node: "Type[Element]") -> None:
     inside ``docutils_namespace()`` to prevent side-effects.
     """
     if not hasattr(nodes.GenericNodeVisitor, 'visit_' + node.__name__):
-        nodes._add_node_class_names([node.__name__])
+        nodes._add_node_class_names([node.__name__])  # type: ignore
         additional_nodes.add(node)
 
 
@@ -187,7 +187,7 @@ class sphinx_domains:
     def __enter__(self) -> None:
         self.enable()
 
-    def __exit__(self, exc_type: "Type[Exception]", exc_value: Exception, traceback: Any) -> bool:  # NOQA
+    def __exit__(self, exc_type: "Type[Exception]", exc_value: Exception, traceback: Any) -> bool:  # type: ignore # NOQA
         self.disable()
         return False
 
