@@ -1,21 +1,18 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.writers.xml
     ~~~~~~~~~~~~~~~~~~
 
     Docutils-native XML and pseudo-XML writers.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from typing import TYPE_CHECKING
-
-from docutils import writers
 from docutils.writers.docutils_xml import Writer as BaseXMLWriter
 
-if TYPE_CHECKING:
-    from typing import Any, Tuple  # NOQA
+if False:
+    # For type annotation
+    from typing import Any  # NOQA
     from sphinx.builders import Builder  # NOQA
 
 
@@ -23,7 +20,7 @@ class XMLWriter(BaseXMLWriter):
 
     def __init__(self, builder):
         # type: (Builder) -> None
-        BaseXMLWriter.__init__(self)
+        super().__init__()
         self.builder = builder
         self.translator_class = self.builder.get_translator_class()
 
@@ -34,23 +31,23 @@ class XMLWriter(BaseXMLWriter):
             self.builder.env.config.xml_pretty
         self.document.settings.xml_declaration = True
         self.document.settings.doctype_declaration = True
-        return BaseXMLWriter.translate(self)
+        return super().translate()
 
 
-class PseudoXMLWriter(writers.Writer):
+class PseudoXMLWriter(BaseXMLWriter):
 
     supported = ('pprint', 'pformat', 'pseudoxml')
     """Formats this writer supports."""
 
     config_section = 'pseudoxml writer'
-    config_section_dependencies = ('writers',)  # type: Tuple[unicode]
+    config_section_dependencies = ('writers',)
 
     output = None
     """Final translated form of `document`."""
 
     def __init__(self, builder):
         # type: (Builder) -> None
-        writers.Writer.__init__(self)
+        super().__init__()
         self.builder = builder
 
     def translate(self):
@@ -58,6 +55,6 @@ class PseudoXMLWriter(writers.Writer):
         self.output = self.document.pformat()
 
     def supports(self, format):
-        # type: (unicode) -> bool
+        # type: (str) -> bool
         """This writer supports all format-specific elements."""
         return True
